@@ -28,12 +28,13 @@ import (
 var CMDQueryDesktop = cli.Command{
 	Name:  "querydesktop",
 	Usage: `pkgname`,
-	Action: func(ctx *cli.Context) {
+	Action: func(ctx *cli.Context) error {
 		querydesktop.InitDB()
 		if ctx.NArg() != 1 {
-			cli.ShowAppHelp(ctx)
-			return
+			_ = cli.ShowAppHelp(ctx)
+			return nil
 		}
 		fmt.Println(querydesktop.QueryDesktopFile(ctx.Args()[0]))
+		return nil
 	},
 }
